        SUBROUTINE CLUST4(
     1                  DATA8,DATA10,NMAX,
     2
     3                  XXCLST,ICLU,KK,NUMCLU)
                                              
C---------------------------------------------------------------------
C       CONSTRUCT CLUSTERS OF RADIANCE DATA
C       USING A BI-VARIATE 1-DIMENSIONAL HISTOGRAM TECHNIQUE
C       ROSSOW, ET AL.,1985: INTERCOMPARISON OF CLOUD CLASSIFICATION
C                            ALGORITHMS., J. CLIM. APPL. MET., P877.
C
C       INPUT ARGUMENTS:
C                  DATA8     1D ARRAY OF CHANNEL 8 RADIANCE DATA
C                  DATA10    1D ARRAY OF CHANNEL 10 RADIANCE DATA
C                  NMAX      SCALAR NUMBER OF RADIANCE DATA VALUES
C
C       OUPUT ARGUMENTS:
C                  XXCLST    2D ARRAY CONTAINING CLUSTER INFO.
C                            1ST DIMENSION CORRESPONDS TO CLUSTER NO.
C                            SECOND DIMENSION CONTAINS EIGHT VALUES
C                            1 - CH8 RADIANCE OF CLUSTER CENTER
C                            2 - STD DEV OF LEFT CH8 GAUSSIAN
C                            3 - STD DEV OF RIGHT CH8 GAUSSIAN
C                            4 - CH10 RADIANCE OF CLUSTER CENTER
C                            5 - STD DEV OF LEFT CH10 GAUSSIAN
C                            6 - STD DEV OF RIGHT CH10 GAUSSIAN
C                            7 - NO. PIXELS BELONGING TO CLUSTER
C                            8 - CLUSTER TO MERGE WITH
C                            3RD DIMENSION IS FOR MERGED VS UNMERGED
C                  ICLU      2D ARRAY CONTAINING CLUSTER INFORMATION
C                            FOR MERGED/UNMERGED CLUSTERS FOR EACH PT
C                  KK        NUMBER OF CLUSTERS FOUND
C                  NUMCLU    NUMBER OF CLUSTERS FOUND(INC. MERGING)
C
C       COMMON BLOCKS:
C                  NEOM
C                  NEOM      OUTPUT WRITE SWITCH
C                  TUNE
C                  NSLOT     NO. SLOTS IN CH8 HISTOGRAM
C                  NSLOT2    NO. SLOTS IN CH10 HISTOGRAM
C                  NCLUST    NO. CLUSTERS TO FIND
C                  IFLG8     FLAG WHICH INSTRUCTS PROGRAM TO FIT
C                            DIMENSIONS OF CH8 HISTOGRAM TO ....
C                            0 - COMPUTE MIN/MAX DATA RANGE AND USE TO
C                                FIND DISTANCE BETWEEN HISTOGRAM SLOTS
C                            1 - USE UNIT DISTANCE BETWEEN HIST. SLOTS
C                  IFLG10    SAME AS IFLG8 BUT FOR CH10
C                  ISMO8     NO OF POINT SMOOTHER TO USE ON CH8 HIST.
C                  ISMO10    NO OF POINT SMOOTHER TO USE ON CH10 HIST.
C                  ISORT     FIRST CLUSTER LOCATING STRATEGY (CH8)
C                            1 - FIND LARGEST CLUSTER FIRST
C                            2 - FIND WARMEST CLUSTER FIRST
C                  NHOT      IF ISORT=2, SET MO. POINTS OF WARM END OF
C                            CH8 HISTOGRAM TO SEARCH FOR FIRST PEAK
C                  ICOFF1    FIRST POINT SELECTION CUTOFF (X10)
C                            ALL POINTS WITHIN ICOFF1 TIMES THE STD DEV
C                            FROM THE HISTOGRAM PEAK WILL BE SELECTED
C                            AS PART OF THE CLUSTER
C                  ICOFF2    SECOND POINT SELECTION CUTOFF (X10)
C                            POINTS BETWEEN ICOFF2 TIMES THE STD DEV
C                            AND THE FIRST CUTOFF WHICH DO NOT TOTAL
C                            GREATER THAN THE GAUSSIAN DISTRIBUTION
C                            WILL BE SELECTED AS PART OF THE CLUSTER
C                  IH2O      TYPE OF CURVE FIT METHOD TO USE
C                            USED BY DRIVING ROUTINE
C                  IKNOB     POSSIBLE VARIANTS ON IH2O METHODS
C                            USED BY DRIVING ROUTINE
C                  ILWOUT    LWFILE OUTPUT SWITCH
C                            1 - DO NOT WRITE LWFILE
C                            2 - WRITE RESULTS TO LWFILE XXSJN7
C                  ISDLO     LOWER LIMIT ON NORMALIZED STD. DEVS WHICH
C                            DESCRIBE GAUSSIAN WHICH IS FIT TO HISTOGRAM
C                  ISDHI     UPPER LIMIT ON NORMALIZED STD. DEVS WHICH
C                            DESCRIBE GAUSSIAN WHICH IS FIT TO HISTOGRAM
C                  IMRG      MULTIPLIER FOR MERGING CRITERION
C                            ALL NEW CLUSTERS WITHIN IMRG TIMES THE STD.
C                            DEV. FROM AN EARLIER CLUSTER CENTER
C                            ARE MERGED INTO THAT EARLIER CLUSTER
C
C       SUBROUTINES CALLED:
C                  HISTM     BUILD HISTOGRAM
C                  FIT2      FIT GAUSSIAN TO HISTOGRAM PEAK
C                  CURVE     COMPUTE HALF-GAUSSIAN FUNCTIONS
C
C       FUNCTIONS CALLED:
C                  IHSTSL    RETURN HISTOGRAM SLOT WHERE DATA VALUE
C                            RESIDES
C
C
C---------------------------------------------------------------------
                                                                      
        PARAMETER     (MMAX=1025)
                                 
        DIMENSION     DATA8(MMAX),DATA10(MMAX)
        DIMENSION     DATA(MMAX)
        DIMENSION     ISLOT(MMAX),IHIST(MMAX)
        DIMENSION     IDIST(MMAX)
        DIMENSION     IHIST2(MMAX),ISLOT2(MMAX)
        DIMENSION     JSLOT(MMAX),KSLOT(MMAX)
        DIMENSION     IMAP(MMAX)
        DIMENSION     ICLU(MMAX,2)
        DIMENSION     XXCLST(20,8,2)
                                    
        CHARACTER*80  CTEXT
                           
        COMMON/NEOM/  NEOM
        COMMON/DEBUG/ KBUG
        COMMON/TUNE/  NSLOT,NSLOT2,NCLUST,IFLG8,IFLG10,
     1                ISMO8,ISMO10,ISORT,ICOFF1,ICOFF2,
     2                IH2O,IKNOB,ILWOUT,NHOT,ISDLO,ISDHI,
     3                IMRG
        REAL*4        COFF1,COFF2
                                 
C       ANOTHER OUTPUT WRITE SWITCH
        NPRT = IKWP('CLUS',1,0)
                               
C       CONVERT POINT SELECTION CUTOFF COEFFICIENTS TO REAL AND SCALE
        COFF1 = FLOAT(ICOFF1)*0.1
        COFF2 = FLOAT(ICOFF2)*0.1
                                 
        IF(NEOM.EQ.14)THEN
C         DUMP CONTENTS OF COMMON/TUNE/ TO SCREEN
          WRITE(CTEXT,2)NSLOT,NSLOT2,NCLUST,IFLG8,IFLG10
          CALL SDEST(CTEXT,0)
          WRITE(CTEXT,2)ISMO8,ISMO10,ISORT,INT(COFF1),INT(COFF2)
          CALL SDEST(CTEXT,0)
          WRITE(CTEXT,2)IH2O,IKNOB,ILWOUT,NHOT,ISDLO,ISDHI,IMRG
          CALL SDEST(CTEXT,0)
          CALL SPFLSH
2         FORMAT(6(1X,I6))
        ENDIF
             
C       INITIALIZE XXCLST TO ZERO
        DO 3 I=1,20
        DO 3 J=1,8
        DO 3 K=1,2
          XXCLST(I,J,K) = 0.
3       CONTINUE
                
C       INITIALIZE CLUSTERING WORKSPACE TO ZERO
        DO 5 I=1,MMAX
          KSLOT(I) = 0
          DO 5 J=1,2
            ICLU(I,J) = 0
5       CONTINUE
                
        NUMCLU = 1
                  
C       ******************************************
C       BEGIN MAIN CLUSTER IDENTIFICATION LOOP
C       ******************************************
                                                  
        DO 500 KK=1,NCLUST
                          
C       INITIALIZE CLUSTERING WORKSPACE ARRAY TO ZERO
        DO 20 I=1,NMAX
                JSLOT(I)=0
20      CONTINUE
                
C       LOAD DATA WITH CH8 RADIANCE DATA WHICH HAS YET TO
C       BE ASSIGNED TO A CLUSTER
        ICT = 1
        DO 22 I=1,NMAX
          IF(KSLOT(I).EQ.0)THEN
                DATA(ICT) = DATA8(I)
C               NOTE ORIGINAL LOCATION OF DATA WITHIN FOV
                IMAP(ICT) = I
                ICT = ICT + 1
          ENDIF
22      CONTINUE
        ICT = ICT - 1
        IF(NEOM.EQ.9)THEN
          WRITE(CTEXT,*)ICT,' DATA POINTS USED IN CH8 HISTOGRAM'
          CALL SDEST(CTEXT,0)
          CALL SPFLSH
        ENDIF
             
C       BUILD CH8 RADIANCE HISTOGRAM
24      CALL HISTM(
     1          DATA(1),ICT,NSLOT,IFLG8,
     2
     3          ISLOT,IHIST,XMIN,XDIF)
        IF(NPRT.EQ.1)THEN
C         OUTPUT HISTOGRAM
          IAAA = 1
          IBBB = 20
8881      CONTINUE
            WRITE(CTEXT,25)(IHIST(J),J=IAAA,IBBB)
            CALL SDEST(CTEXT,0)
            IAAA = IAAA+20
            IBBB = IBBB+20
          IF(IBBB.LT.NSLOT)GOTO 8881
25        FORMAT(20(1X,I2))
          CALL SDEST(' ',0)
          CALL SPFLSH
        ENDIF
             
        IF(XDIF.EQ.0.)THEN
C         EXIT MAIN CLUSTER ID LOOP
C         HISTOGRAM COULD NOT BE BUILT
          GOTO 777
        ENDIF
             
C       FIT HALF-GAUSSIAN CURVES TO PEAK IN HISTOGRAM
C       AFTER SMOOTHING
        IF(ISORT.EQ.2)THEN
C         FIND WARM (CH8) CLUSTER FIRST; THEN SEARCH FOR MAX
C         FREQUENCY WHEN FINDING REMAINING CLUSTERS
          IF(KK.EQ.1)THEN
C           TELL FIT TO FIND CLUSTER AROUND PEAK ON
C           THE WARM END OF THE HISTOGRAM
            IJOB = 2
          ELSE
C           TELL FIT TO FIND CLUSTER AROUND MAX HISTOGRAM FREQUENCY
            IJOB = 1
          ENDIF
        ELSE
C         FIND ALL CLUSTERS BASED ON MAX FREQUENCY
          IJOB = 1
        ENDIF
        CALL FIT2(
     1          IHIST,NSLOT,XMIN,XDIF,ISMO8,IJOB,NHOT,
     2
     3          IIND,FAVE,VARL,VARR)
                                    
        IF(VARL.EQ.0..OR.VARR.EQ.0.)THEN
C         EXIT MAIN CLUSTER ID LOOP
C         GAUSSIAN FIT FAILED
          GOTO 777
        ENDIF
             
C       CONSTRAIN THE VARIANCE TO HAVE AN UPPER AND LOWER LIMIT
        XLIMHI = (FLOAT(ISDHI)*XDIF*0.01)**2.
        XLIMLO = (FLOAT(ISDLO)*XDIF*0.01)**2.
        IF(VARL.GT.XLIMHI) VARL = XLIMHI
        IF(VARL.LT.XLIMLO) VARL = XLIMLO
        IF(VARR.GT.XLIMHI) VARR = XLIMHI
        IF(VARR.LT.XLIMLO) VARR = XLIMLO
                                        
C       COMPUTE FULL GAUSSIAN CURVE
        CALL CURVE(
     1          NSLOT,XMIN,XDIF,IIND,FAVE,VARL,VARR,
     2
     3          IDIST)
                      
C       COMPUTE RADIANCE AT GAUSSIAN PEAK AND STANDARD DEVIATIONS
C       OF CURVES ON EACH SIDE OF PEAK
        RLO = XMIN + FLOAT(IIND-1)*XDIF
        RAVE = (RLO + (RLO+XDIF))/2.
        SDEVL = SQRT(VARL)/XDIF
        SDEVR = SQRT(VARR)/XDIF
                               
C       COMPUTE INDICES CORRESPONDING TO POINT SELECTION THRESHHOLDS
        IND1 = IIND - COFF2*SDEVL
        IND2 = IIND - COFF1*SDEVL
        IND3 = IIND + COFF1*SDEVR
        IND4 = IIND + COFF2*SDEVR
                                 
C       SELECT POINTS FOR USE IN CH10 HISTOGRAM
        DO 100 I = 1,ICT
          IF(NEOM.EQ.15)THEN
C           DUMP CONTENTS OF CLUSTERING WORKSPACE
            WRITE(CTEXT,*)I,IMAP(I),JSLOT(IMAP(I)),ISLOT(I)
            CALL SDEST(CTEXT,0)
            CALL SPFLSH
          ENDIF
          IF(ISLOT(I).GE.IND2.AND.ISLOT(I).LE.IND3)THEN
C            SELECT ALL POINTS WITHIN 1 S.D. OF PEAK
             JSLOT(IMAP(I)) = 1
          ELSE IF(ISLOT(I).GE.IND1.AND.ISLOT(I).LT.IND2)THEN
C            BETWEEN 1 S.D. AND 3 S.D. SELECT ONLY AS MANY POINTS
C            AS SUGGESTED BY GAUSSIAN FIT
             IF( IDIST(ISLOT(I)) .GT. 0 )THEN
                JSLOT(IMAP(I)) = 1
                IDIST(ISLOT(I)) = IDIST(ISLOT(I)) - 1
             ENDIF
          ELSE IF(ISLOT(I).GT.IND3.AND.ISLOT(I).LE.IND4)THEN
C            BETWEEN 1 S.D. AND 3 S.D. SELECT ONLY AS MANY POINTS
C            AS SUGGESTED BY GAUSSIAN FIT
             IF( IDIST(ISLOT(I)) .GT. 0 )THEN
                JSLOT(IMAP(I)) = 1
                IDIST(ISLOT(I)) = IDIST(ISLOT(I)) - 1
             ENDIF
          ENDIF
          IF(ISLOT(I).EQ.IIND.AND.JSLOT(IMAP(I)).NE.1)THEN
C               MAKE SURE ALL POINTS CORRESPONDING TO PEAK ARE CHOSEN
                JSLOT(IMAP(I))=1
                IDIST(ISLOT(I)) = IDIST(ISLOT(I)) - 1
          ENDIF
100     CONTINUE
                
        ICNT = 1
        DO 110 I=1,NMAX
          IF(JSLOT(I).EQ.1)THEN
C               USE DATA POINT FOR CH10 HISTOGRAM
                DATA(ICNT) = DATA10(I)
C               CONTINUE TO MAP DATA POINTS ORIGINAL LOCATION IN FOV
                IMAP(ICNT) = I
                ICNT = ICNT + 1
          ENDIF
110     CONTINUE
        ICNT = ICNT-1
                     
C       SAVE RADIANCE PEAK AND STD. DEVS. FOR CH8 GAUSSIAN
        XXCLST(NUMCLU,1,1) = RAVE
        XXCLST(NUMCLU,2,1) = SDEVL
        XXCLST(NUMCLU,3,1) = SDEVR
        XXCLST(KK,1,2) = RAVE
        XXCLST(KK,2,2) = SDEVL
        XXCLST(KK,3,2) = SDEVR
                              
        IF(KBUG.EQ.2)THEN
          CALL SDEST(' ',0)
          WRITE(CTEXT,*)' XMIN   ',XMIN  ,' XDIF   ',XDIF
          CALL SDEST(CTEXT,0)
          WRITE(CTEXT,*)' FAVE   ',FAVE  ,' RAVE   ',RAVE
          CALL SDEST(CTEXT,0)
          WRITE(CTEXT,*)' SDEVL  ',SDEVL ,' SDEVR  ',SDEVR
          CALL SDEST(CTEXT,0)
          CALL SDEST(' ',0)
        ENDIF
             
        IF(NEOM.EQ.9)THEN
          WRITE(CTEXT,*)ICNT,' DATA POINTS USED IN CH10 HISTOGRAM'
          CALL SDEST(CTEXT,0)
          CALL SPFLSH
        ENDIF
             
C       BUILD CH10 RADIANCE HISTOGRAM
120     CALL HISTM(
     1          DATA(1),ICNT,NSLOT2,IFLG10,
     2
     3          ISLOT2,IHIST2,XMIN2,XDIF2)
        IF(NPRT.EQ.1)THEN
C         OUTPUT HISTOGRAM
          IAAA = 1
          IBBB = 20
8882      CONTINUE
            WRITE(CTEXT,25)(IHIST2(J),J=IAAA,IBBB)
            CALL SDEST(CTEXT,0)
            IAAA = IAAA+20
            IBBB = IBBB+20
          IF(IBBB.LT.NSLOT2)GOTO 8882
          CALL SDEST(' ',0)
          CALL SPFLSH
        ENDIF
             
        IF(XDIF2.EQ.0.)THEN
C         EXIT MAIN CLUSTER ID LOOP
C         HISTOGRAM COULD NOT BE BUILT
          GOTO 777
        ENDIF
             
C       FIT HALF-GAUSSIAN CURVES TO PEAK IN CH10 HISTOGRAM
C       AFTER SMOOTHING
        IJOB = 1
        CALL FIT2(
     1          IHIST2,NSLOT2,XMIN2,XDIF2,ISMO10,IJOB,NHOT,
     2
     3          IIND,FAVE,VARL,VARR)
                                    
        IF(VARL.EQ.0..OR.VARR.EQ.0.)THEN
C         EXIT MAIN CLUSTER ID LOOP
C         GAUSSIAN FIT FAILED
          GOTO 777
        ENDIF
             
C       CONSTRAIN THE VARIANCE TO HAVE AN UPPER AND LOWER LIMIT
        XLIMHI = (FLOAT(ISDHI)*XDIF2*0.01)**2.
        XLIMLO = (FLOAT(ISDLO)*XDIF2*0.01)**2.
        IF(VARL.GT.XLIMHI) VARL = XLIMHI
        IF(VARL.LT.XLIMLO) VARL = XLIMLO
        IF(VARR.GT.XLIMHI) VARR = XLIMHI
        IF(VARR.LT.XLIMLO) VARR = XLIMLO
                                        
C       COMPUTE FULL GAUSSIAN CURVE
        CALL CURVE(
     1          NSLOT2,XMIN2,XDIF2,IIND,FAVE,VARL,VARR,
     2
     3          IDIST)
                      
C       COMPUTE RADIANCE AT GAUSSIAN PEAK AND STANDARD DEVIATIONS
C       OF CURVES ON EACH SIDE OF PEAK
        RLO = XMIN2 + FLOAT(IIND-1)*XDIF2
        RAVE = (RLO + (RLO+XDIF2))/2.
        SDEVL = SQRT(VARL)/XDIF2
        SDEVR = SQRT(VARR)/XDIF2
                                
C       COMPUTE INDICES CORRESPONDING TO SELECTION THRESHHOLDS
        IND1 = IIND - COFF2*SDEVL
        IND2 = IIND - COFF1*SDEVL
        IND3 = IIND + COFF1*SDEVR
        IND4 = IIND + COFF2*SDEVR
                                 
C       SELECT POINTS FOR CLUSTER
        DO 200 I = 1,ICNT
            IF(ISLOT2(I).GE.IND2.AND.ISLOT2(I).LE.IND3)THEN
C             CHOOSE ALL POINTS WITHIN 1 S.D. OF CH10 HISTOGRAM PEAK
              ICLU(IMAP(I),1) = -1*NUMCLU
              ICLU(IMAP(I),2) = -1*KK
              KSLOT(IMAP(I)) = 1
            ELSE IF(ISLOT2(I).GE.IND1.AND.ISLOT2(I).LT.IND2)THEN
C             BETWEEN 1 S.D. AND 3 S.D. SELECT ONLY AS MANY POINTS
C             AS SUGGESTED BY GAUSSIAN FIT
              IF( IDIST(ISLOT2(I)) .GT. 0 )THEN
                ICLU(IMAP(I),1) = -1*NUMCLU
                ICLU(IMAP(I),2) = -1*KK
                KSLOT(IMAP(I)) = 1
                IDIST(ISLOT2(I)) = IDIST(ISLOT2(I)) - 1
              ENDIF
            ELSE IF(ISLOT2(I).GT.IND3.AND.ISLOT2(I).LE.IND4)THEN
C             BETWEEN 1 S.D. AND 3 S.D. SELECT ONLY AS MANY POINTS
C             AS SUGGESTED BY GAUSSIAN FIT
              IF( IDIST(ISLOT2(I)) .GT. 0 )THEN
                ICLU(IMAP(I),1) = -1*NUMCLU
                ICLU(IMAP(I),2) = -1*KK
                KSLOT(IMAP(I)) = 1
                IDIST(ISLOT2(I)) = IDIST(ISLOT2(I)) - 1
              ENDIF
            ENDIF
            IF(ISLOT2(I).EQ.IIND.AND.KSLOT(IMAP(I)).NE.1)THEN
C               MAKE SURE ANY POINTS IN CH10 HISTOGRAM PEAK ARE TAKEN
                ICLU(IMAP(I),1) = -1*NUMCLU
                ICLU(IMAP(I),2) = -1*KK
                KSLOT(IMAP(I)) = 1
                IDIST(ISLOT2(I)) = IDIST(ISLOT2(I)) - 1
            ENDIF
200     CONTINUE
                
C       COUNT TOTAL NO. OF POINTS IN FINISHED CLUSTER
        ICNT = 1
        DO 205 I=1,NMAX
            IF(ICLU(I,1).EQ.(-1*NUMCLU))THEN
                ICNT = ICNT + 1
            ENDIF
205     CONTINUE
        ICNT = ICNT-1
                     
C       SAVE RADIANCE AND STANDARD DEVS OF CH10 GAUSSIAN FIT
        XXCLST(NUMCLU,4,1) = RAVE
        XXCLST(NUMCLU,5,1) = SDEVL
        XXCLST(NUMCLU,6,1) = SDEVR
        XXCLST(NUMCLU,7,1) = FLOAT(ICNT)
        XXCLST(KK,4,2) = RAVE
        XXCLST(KK,5,2) = SDEVL
        XXCLST(KK,6,2) = SDEVR
        XXCLST(KK,7,2) = FLOAT(ICNT)
                                    
        IF(KBUG.EQ.2)THEN
          CALL SDEST(' ',0)
          WRITE(CTEXT,*)' XMIN2   ',XMIN2  ,' XDIF2   ',XDIF2
          CALL SDEST(CTEXT,0)
          WRITE(CTEXT,*)' FAVE   ',FAVE  ,' RAVE   ',RAVE
          CALL SDEST(CTEXT,0)
          WRITE(CTEXT,*)' SDEVL  ',SDEVL ,' SDEVR  ',SDEVR
          CALL SDEST(CTEXT,0)
          CALL SDEST(' ',0)
        ENDIF
             
210     FORMAT(10X,8(1X,F8.2))
                              
C       MERGE (IGNORE) THIS CLUSTER OF POINTS IF IT'S CENTER IS
C       WITHIN IMRG STANDARD DEVIATIONS OF ANY PREVIOUSLY DEFINED
C       CLUSTER'S CENTER IN 2D RADIANCE SPACE
        DO 300 KLOOP = 1,NUMCLU-1
          M1 = XXCLST(KLOOP,1,1)-FLOAT(IMRG)*0.1*XXCLST(KLOOP,2,1)
          M2 = XXCLST(KLOOP,1,1)+FLOAT(IMRG)*0.1*XXCLST(KLOOP,3,1)
          N1 = XXCLST(KLOOP,4,1)-FLOAT(IMRG)*0.1*XXCLST(KLOOP,5,1)
          N2 = XXCLST(KLOOP,4,1)+FLOAT(IMRG)*0.1*XXCLST(KLOOP,6,1)
          MC = XXCLST(NUMCLU,1,1)
          NC = XXCLST(NUMCLU,4,1)
          IF(MC.GT.M1.AND.MC.LT.M2.AND.
     1       NC.GT.N1.AND.NC.LT.N2)THEN
                XXCLST(KK,8,2) = FLOAT(KLOOP)
                DO 250 JI=1,NMAX
                  IF(ICLU(JI,1).EQ.(-1*NUMCLU))THEN
                    ICLU(JI,1) = -1*KLOOP
                    XXCLST(KLOOP,7,1) = XXCLST(KLOOP,7,1) + 1.
                  ENDIF
250             CONTINUE
                NUMCLU = NUMCLU - 1
                GOTO 310
          ENDIF
300     CONTINUE
310     CONTINUE
                
        NUMCLU = NUMCLU + 1
                           
500     CONTINUE
                
C       ****************************************
C       END MAIN CLUSTER IDENTIFICATION LOOP
C       ****************************************
                                                
777     CONTINUE
                
        KK = KK - 1
        NUMCLU = NUMCLU - 1
                           
C       PRINT RADIANCE PEAKS, STD. DEVS AND PIXEL TOTALS FOR EACH
C       SELECTED CLUSTER
        DO 800 I=1,NUMCLU
CCC             WRITE(6,*)
CCC             WRITE(6,*)'CLUSTER ',I
                INO = 1
                DO 790 J=1,NMAX
                  IF(ICLU(J,1).EQ.(-1*I))THEN
CCC                     WRITE(CTEXT,789)INO,DATA8(J),DATA10(J)
789                     FORMAT(1X,I4,2(1X,F8.2))
CCC                     CALL SDEST(CTEXT,0)
                        INO = INO + 1
                  ENDIF
790             CONTINUE
                INO = INO - 1
                XXCLST(I,7,1)=FLOAT(INO)
800     CONTINUE
                
CCC     WRITE(6,*)
                  
        RETURN
        END
           
C###############################################################
                                                                
        SUBROUTINE CURVE(
     1                  NSLOT,XMIN,XDIF,IIND,FAVE,VARL,VARR,
     2
     3                  IDIST)
                              
C-------------------------------------------------------------------
C       CALCULATE GAUSSIAN CURVE GIVEN ....
C               NSLOT   HISTOGRAM SLOTS TO FILL WITH CURVE VALUES
C               XMIN    MINIMUM DATA VALUE IN HISTOGRAM
C               XDIF    DATA VALUE INCREMENT BETWEEN HISTOGRAM SLOTS
C               IIND    INDEX OF HISTOGRAM SLOT WITH FREQUENCY PEAK
C               FAVE    FREQUENCY CORRESPONDING TO PEAK
C               VARL    VARIANCE DEFINING LEFT SIDE OF CURVE
C               VARR    VARIANCE DEFINING RIGHT SIDE OF CURVE
C------------------------------------------------------------------
                                                                   
        PARAMETER (MMAX=1025)
                             
        DIMENSION IDIST(MMAX)
                             
        DO 20 I=1,MMAX
                IDIST(I) = 0
20      CONTINUE
                
        RLO = XMIN + FLOAT(IIND-1)*XDIF
        RAVE = (RLO + (RLO+XDIF))/2.
        DO 40 I=1,NSLOT
          IF(I.LE.IIND)THEN
                VAR = VARL
          ELSE
                VAR = VARR
          ENDIF
          RLO = XMIN + FLOAT(I-1)*XDIF
          R = (RLO + (RLO+XDIF))/2.
          XDIST = FGAUSS(FAVE,R,RAVE,VAR)
          IDIST(I) = XDIST
40      CONTINUE
                
CCC     WRITE(6,*)'*** EXITING CURVE ***'
                                         
        RETURN
        END
           
C###################################################################
                                                                    
        SUBROUTINE FIT2(
     1                  IHIST,NSLOT,XMIN,XDIF,ISMOO,IJOB,NHOT,
     2
     3                  IIND,FAVE,VARL,VARR)
                                            
C-------------------------------------------------------------------
C       GIVEN ARGUMENTS DEFINING A HISTOGRAM WHICH ARE ...
C               IHIST   HISTOGRAM FREQUENCIES
C               NSLOT   NO. DATA SLOTS IN HISTOGRAM
C               XMIN    MINIMUM DATA VALUE IN HISTOGRAM
C               XDIF    DATA VALUE RANGE OF EACH HISTOGRAM SLOT
C               ISMOO   INTEGER FLAG FOR SMOOTH/NO-SMOOTH
C                       IF 0 - DO NOT APPLY SMOOTHER
C                       IF # - SMOOTH USING # POINTS
C       APPLY A FIVE POINT SMOOTHER TO THE HISTOGRAM.  FIND THE
C       PEAK FREQUENCY IN THE SMOOTHED HISTOGRAM (SEARCHING FROM
C       THE HIGH DATA VALUE END) AND FIT A GAUSSIAN DISTRIBUTION
C       TO EACH HALF OF THAT PEAK.  RETURN ...
C               IIND    HISTOGRAM SLOT CORRESPONDING TO PEAK
C               FAVE    FREQUENCY OF PEAK
C               VARL    VARIANCE DEFINING LEFT SIDE OF CURVE
C               VARR    VARIANCE DEFINING RIGHT SIDE OF CURVE
C------------------------------------------------------------------
                                                                   
        PARAMETER (MMAX=1025)
                             
        DIMENSION IHIST(MMAX)
        DIMENSION IDIST(MMAX),XDIST(MMAX)
                                         
C       FLOAT HISTOGRAM INTO XDIST
        DO 35 I=1,NSLOT
                XDIST(I) = FLOAT(IHIST(I))
35      CONTINUE
                
C       APPLY ISMOO-POINT SMOOTHER TO HISTOGRAM
        PTS = FLOAT(ISMOO)
        IF(ISMOO.NE.0)CALL SMOOFT(XDIST,NSLOT,PTS)
                                                  
C       TRUNCATE SMOOTHED HISTOGRAM
        DO 40 I=1,NSLOT
                IDIST(I) = XDIST(I)
40      CONTINUE
C       OUTPUT SMOOTHED HISTOGRAM
CCC     WRITE(6,43)(IDIST(J),J=1,NSLOT)
43      FORMAT(50(18(1X,I3)/))
CCC     WRITE(6,*)
                  
        IF(IJOB.EQ.1)THEN
C         FIND MAX FREQUENCY IN SMOOTHED HISTOGRAM
C         CHECK FROM WARM END FIRST
          XAVE = 0.
          DO 45 I=NSLOT,1,-1
            IF(XDIST(I).GT.XAVE)THEN
                XAVE = XDIST(I)
                IIND = I
            ENDIF
45        CONTINUE
        ELSE IF(IJOB.EQ.2)THEN
C         FIND FREQUENCY OF WARMEST CLUSTER
C         CHECK FROM WARM END FIRST
          XAVE = 0.
          ISTART = 0
          DO 46 I=NSLOT,1,-1
            IF(XDIST(I).GE.1.)ISTART=ISTART+1
            IF(XDIST(I).GT.XAVE)THEN
                XAVE = XDIST(I)
                IIND = I
            ENDIF
            IF(ISTART.GE.NHOT)GOTO 47
46        CONTINUE
47        CONTINUE
        ENDIF
             
CCC     WRITE(6,*)'MAX FREQUENCY IN SLOT ',IIND,' IS ',XAVE
CCC     WRITE(6,*)
                  
C       LOOP TO FIT GAUSSIANS TO BOTH SIDES OF PEAK
        DO 250 JJ=1,2
                     
          IF(JJ.EQ.1)THEN
C               FIT GAUSSIAN TO LEFT SIDE OF PEAK
                IA = IIND-3
                IB = IIND-1
          ELSE
C               FIT GAUSSIAN TO RIGHT SIDE OF PEAK
                IA = IIND+1
                IB = IIND+3
          ENDIF
          IF(IA.LT.1)IA=1
          IF(IB.GT.NSLOT)IB=NSLOT
                                 
C         AVERAGE GAUSSIAN CURVES 3 POINTS EACH SIDE OF PEAK
C         TO OBTAIN AVERAGE VARIANCE OF THOSE CURVES
          ILOOP = 0
          XVAR = 0.
          DO 100 INDEX=IA,IB
CCC         WRITE(6,*)'*** ENTERING TPGAUS ***'
            CALL TPGAUS(
     1          IIND,INDEX,XDIST,XMIN,XDIF,
     2
     3          VAR)
            IF(VAR.GT.0.)THEN
              ILOOP = ILOOP + 1
              XVAR = XVAR + VAR
CCC           WRITE(6,*)'   VARIANCE = ',VAR
            ENDIF
100       CONTINUE
          IF(ILOOP.NE.0)THEN
            XVAR = XVAR/FLOAT(ILOOP)
          ELSE
CCC         WRITE(6,*)'ILOOP 0 IN FIT'
CCC         WRITE(6,*)
            XVAR = 0.
          ENDIF
ccc       WRITE(6,*)'   AVG VARIANCE = ',XVAR
                                             
C         STORE AVG VARIANCES FOR EACH SIDE OF CURVE
          IF(JJ.EQ.1)THEN
            VARL = XVAR
          ELSE
            VARR = XVAR
          ENDIF
               
250     CONTINUE
                
C       STORE FREQUENCY OF PEAK
        FAVE = XDIST(IIND)
                          
CCC     WRITE(6,*)'*** EXITING SUBROUTINE FIT ***'
                                                  
        RETURN
        END
           
C#######################################################################
                                                                        
        FUNCTION FGAUSS(FAVE,R,RAVE,VAR)
                                        
C-----------------------------------------------------------------
C       COMPUTE FREQUENCY OF GAUSSIAN FUNCTION GIVEN ...
C               FAVE    FREQUENCY AT PEAK OF DISTRIBUTION
C               R       DATA VALUE (POSITION RELATIVE TO PEAK)
C               RAVE    DATA VALUE CORRESPONDING TO PEAK
C               VAR     VARIANCE DESCRIBING CURVE
C-----------------------------------------------------------------
                                                                  
        XVAR = (-1.*(R-RAVE)*(R-RAVE))/(2.*VAR)
        IF(XVAR.LT.-10.)THEN
          FGAUSS = 0.
        ELSE
          FGAUSS = FAVE * EXP(XVAR)
        ENDIF
             
        RETURN
        END
           
C#######################################################################
                                                                        
        SUBROUTINE HISTM(
     1                  DATA,NMAX,NSLOT,IFLAG,
     2
     3                  ISLOT,IHIST,XMIN,XDIF)
                                              
C---------------------------------------------------------------
C       CONSTRUCT A HISTOGRAM GIVEN ....
C               DATA    INPUT REAL ARRAY OF DATA
C               NMAX    NO OF VALUES IN ARRAY DATA
C               NSLOT   NO OF SLOTS DESIRED IN HISTOGRAM
C               IFLAG   RADIANCE DATA (1) OR OTHER (0)
C
C       OUTPUT ....
C               ISLOT   OUTPUT ARRAY MARKING EACH DATA POINT'S
C                       RESULTANT HISTOGRAM SLOT
C               IHIST   OUTPUT ARRAY OF NO OF POINTS IN EACH
C                       HISTOGRAM SLOT (I.E. THE HISTOGRAM)
C               XMIN    MINIMUM DATA VALUE
C               XDIF    VALUE RANGE OF EACH HISTOGRAM SLOT
C---------------------------------------------------------------
                                                                
        DIMENSION  DATA(NMAX)
        DIMENSION  ISLOT(NMAX)
        DIMENSION  IHIST(NMAX)
                              
C       CALCULATE MINIMUM DATA VALUE AND VALUE RANGE OF EACH
C       HISTOGRAM SLOT
        CALL HISTD(
     1          DATA,NMAX,NSLOT,IFLAG,
     2
     3          XMIN,XDIF)
                          
        IF(XDIF.EQ.0.)GOTO 777
                              
C       EMPTY HISTOGRAM SLOTS
        DO 50 I=1,NSLOT
                IHIST(I)=0
50      CONTINUE
                
C       LOOP THROUGH DATA AND FILL HISTOGRAM
        DO 100 I=1,NMAX
                       
C               CALCULATE WHICH HISTOGRAM SLOT DATA POINT FALLS IN
                ISLOT(I) = IHSTSL(DATA(I),XMIN,XDIF)
                                                    
C               ADD POINT TO CORRECT HISTOGRAM SLOT
                IHIST(ISLOT(I)) = IHIST(ISLOT(I)) + 1
                                                     
100     CONTINUE
                
777     CONTINUE
                
        RETURN
        END
           
C###################################################################
                                                                    
        SUBROUTINE HISTD(
     1                  DATA,NMAX,NSLOT,IFLAG,
     2
     3                  XMIN,XDIF)
                                  
C---------------------------------------------------------------
C       DEFINE MINIMUM DATA VALUE AND RANGE OF HISTOGRAM VALUE
C       FOR EACH HISTOGRAM SLOT
C---------------------------------------------------------------
                                                                
        DIMENSION DATA(NMAX)
                            
        IF(IFLAG.EQ.1)THEN
          XMIN = 0.
          XMAX = FLOAT(NSLOT-1)
        ELSE
          XMIN = 9999.
          XMAX = 0.
          DO 10 I=1,NMAX
                IF(DATA(I).LT.XMIN) XMIN=DATA(I)
                IF(DATA(I).GT.XMAX) XMAX=DATA(I)
10        CONTINUE
        ENDIF
             
        XDIF = (XMAX-XMIN)/FLOAT(NSLOT-1)
                                         
CCC     WRITE(6,*)'*** EXITING HISTD ***'
                                         
        RETURN
        END
           
C###################################################################
                                                                    
        FUNCTION IHSTSL(DATA,XMIN,XDIF)
                                       
C-----------------------------------------------------------------
C       COMPUTE HISTOGRAM SLOT IN WHICH DATA VALUE FALLS GIVEN...
C               DATA    THE DATA VALUE
C               XMIN    MINIMUM VALUE OF DATA
C               XDIF    DIFFERENCE IN VALUE ACROSS ONE HISTOGRAM
C                       SLOT
C-----------------------------------------------------------------
                                                                  
                                                                  
        IHSTSL = INT(  (DATA-XMIN)/XDIF )
                                         
        RETURN
        END
           
C#######################################################################
                                                                        
        SUBROUTINE TPGAUS(
     1                  ICEN,IND,XDIST,XMIN,XDIF,
     2
     3                  VAR)
                            
C---------------------------------------------------------------------
C       THIS SUBROUTINE USES THE TWO POINT METHOD TO FIT A GAUSSIAN
C       (NORMAL) DISTRIBUTION TO A HISTOGRAM USING ...
C               ICEN    CENTER HISTOGRAM POINT TO FIT CURVE TO
C               IND     OTHER HISTOGRAM POINT
C               XDIST   HISTOGRAM (REAL*4)
C               XMIN    MINIMUM DATA VALUE OF HISTOGRAM
C               XDIF    DIFFERENCE BETWEEN HISTOGRAM DATA VALUES
C       OUTPUT ...
C               VAR     VARIANCE DESCRIBING GAUSSIAN CURVE
C---------------------------------------------------------------------
                                                                      
        PARAMETER (MMAX=1025)
                             
        DIMENSION       XDIST(MMAX)
                                   
        RLO = XMIN + FLOAT(ICEN-1)*XDIF
        RAVE = (RLO + (RLO+XDIF))/2.
        FAVE = XDIST(ICEN)
                          
        RLO = XMIN + FLOAT(IND-1)*XDIF
        R = (RLO + (RLO+XDIF))/2.
        F = XDIST(IND)
                      
        IF(F.LT.0.)F=0.
        IF(FAVE.LT.0.)FAVE=0.
                             
CCC     WRITE(6,*)RAVE,FAVE,R,F
                               
        IF(F.NE.0..AND.FAVE.NE.0..AND.F.NE.FAVE)THEN
          XNUM = -1.*(R-RAVE)*(R-RAVE)
          XDEN = 2.*ALOG(F/FAVE)
          VAR = XNUM/XDEN
        ELSE
          VAR = 0.
        ENDIF
             
CCC     WRITE(6,*)'*** EXITING TPGAUS ***'
                                          
        RETURN
        END
           
C#######################################################################
                                                                        
      SUBROUTINE SMOOFT(Y,N,PTS)
      PARAMETER(MMAX=1025)
      DIMENSION Y(MMAX)
      M=2
      NMIN=N+2.*PTS
1     IF(M.LT.NMIN)THEN
        M=2*M
      GO TO 1
      ENDIF
      IF(M.GT.MMAX) PAUSE 'MMAX too small'
      CONST=(PTS/M)**2
      Y1=Y(1)
      YN=Y(N)
      RN1=1./(N-1.)
      DO 11 J=1,N
        Y(J)=Y(J)-RN1*(Y1*(N-J)+YN*(J-1))
11    CONTINUE
      IF(N+1.LE.M)THEN
        DO 12 J=N+1,M
          Y(J)=0.
12      CONTINUE
      ENDIF
      MO2=M/2
      CALL REALFT(Y,MO2,1)
      Y(1)=Y(1)/MO2
      FAC=1.
      DO 13 J=1,MO2-1
        K=2*J+1
        IF(FAC.NE.0.)THEN
          FAC=AMAX1(0.,(1.-CONST*J**2)/MO2)
          Y(K)=FAC*Y(K)
          Y(K+1)=FAC*Y(K+1)
        ELSE
          Y(K)=0.
          Y(K+1)=0.
        ENDIF
13    CONTINUE
      FAC=AMAX1(0.,(1.-0.25*PTS**2)/MO2)
      Y(2)=FAC*Y(2)
      CALL REALFT(Y,MO2,-1)
      DO 14 J=1,N
        Y(J)=RN1*(Y1*(N-J)+YN*(J-1))+Y(J)
14    CONTINUE
      RETURN
      END
         
C#######################################################################
                                                                        
      SUBROUTINE REALFT(DATA,N,ISIGN)
      REAL*8 WR,WI,WPR,WPI,WTEMP,THETA
      DIMENSION DATA(*)
      THETA=6.28318530717959D0/2.0D0/DBLE(N)
      C1=0.5
      IF (ISIGN.EQ.1) THEN
        C2=-0.5
        CALL FOUR1(DATA,N,+1)
      ELSE
        C2=0.5
        THETA=-THETA
      ENDIF
      WPR=-2.0D0*DSIN(0.5D0*THETA)**2
      WPI=DSIN(THETA)
      WR=1.0D0+WPR
      WI=WPI
      N2P3=2*N+3
      DO 11 I=2,N/2+1
        I1=2*I-1
        I2=I1+1
        I3=N2P3-I2
        I4=I3+1
        WRS=SNGL(WR)
        WIS=SNGL(WI)
        H1R=C1*(DATA(I1)+DATA(I3))
        H1I=C1*(DATA(I2)-DATA(I4))
        H2R=-C2*(DATA(I2)+DATA(I4))
        H2I=C2*(DATA(I1)-DATA(I3))
        DATA(I1)=H1R+WRS*H2R-WIS*H2I
        DATA(I2)=H1I+WRS*H2I+WIS*H2R
        DATA(I3)=H1R-WRS*H2R+WIS*H2I
        DATA(I4)=-H1I+WRS*H2I+WIS*H2R
        WTEMP=WR
        WR=WR*WPR-WI*WPI+WR
        WI=WI*WPR+WTEMP*WPI+WI
11    CONTINUE
      IF (ISIGN.EQ.1) THEN
        H1R=DATA(1)
        DATA(1)=H1R+DATA(2)
        DATA(2)=H1R-DATA(2)
      ELSE
        H1R=DATA(1)
        DATA(1)=C1*(H1R+DATA(2))
        DATA(2)=C1*(H1R-DATA(2))
        CALL FOUR1(DATA,N,-1)
      ENDIF
      RETURN
      END
         
C#######################################################################
                                                                        
      SUBROUTINE FOUR1(DATA,NN,ISIGN)
      REAL*8 WR,WI,WPR,WPI,WTEMP,THETA
      DIMENSION DATA(*)
      N=2*NN
      J=1
      DO 11 I=1,N,2
        IF(J.GT.I)THEN
          TEMPR=DATA(J)
          TEMPI=DATA(J+1)
          DATA(J)=DATA(I)
          DATA(J+1)=DATA(I+1)
          DATA(I)=TEMPR
          DATA(I+1)=TEMPI
        ENDIF
        M=N/2
1       IF ((M.GE.2).AND.(J.GT.M)) THEN
          J=J-M
          M=M/2
        GO TO 1
        ENDIF
        J=J+M
11    CONTINUE
      MMAX=2
2     IF (N.GT.MMAX) THEN
        ISTEP=2*MMAX
        THETA=6.28318530717959D0/(ISIGN*MMAX)
        WPR=-2.D0*DSIN(0.5D0*THETA)**2
        WPI=DSIN(THETA)
        WR=1.D0
        WI=0.D0
        DO 13 M=1,MMAX,2
          DO 12 I=M,N,ISTEP
            J=I+MMAX
            TEMPR=SNGL(WR)*DATA(J)-SNGL(WI)*DATA(J+1)
            TEMPI=SNGL(WR)*DATA(J+1)+SNGL(WI)*DATA(J)
            DATA(J)=DATA(I)-TEMPR
            DATA(J+1)=DATA(I+1)-TEMPI
            DATA(I)=DATA(I)+TEMPR
            DATA(I+1)=DATA(I+1)+TEMPI
12        CONTINUE
          WTEMP=WR
          WR=WR*WPR-WI*WPI+WR
          WI=WI*WPR+WTEMP*WPI+WI
13      CONTINUE
        MMAX=ISTEP
      GO TO 2
      ENDIF
      RETURN
      END
