      SUBROUTINE GRD2MD(IGRD,IMD,IG1,IG2,IDAY,ITIM,NKYS,CKYS,CGKYS,
     +IGLVS,NLVL,NPRM)
C
C  SUBROUTINE GRD2MD takes data out of a GRID file and places it into
C   an MD file.
C
C  PARAMETERS (all are INPUT):
C
C   IGRD= GRID file number
C   IMD= MD file number
C   IG1= Beginning GRID in GRID file to process
C   IG2= End GRID in GRID file to process
C   IDAY= Date to be put in MD file header
C   ITIM= Time to be put in MD file header
C   NKYS= Number of keys in MD file to fill
C   CKYS= Array of key names in MD file to fill
C   CGKYS= Array of key names from GRID file
C   IGLVS= Array of number of levels for each GRID file parameter
C   NLVL= Array of GRID file parameter levels
C   NPRM= Number of GRID file parameters
C
C      REAL FVAL(200,400,400)
      REAL FVAL(400,400,200)
      INTEGER ISCALE(400),IUNIT(400),LOCS(400),IREC(400),IGLVS(400),
     +HEADER(64),NLVL(400,400),IGOT(400),PLEV,DUMMY(70000)
      CHARACTER*4 CGKYS(400),PARM
      CHARACTER*12 CKYS,CLIT
      COMMON/NAV/FLAT,FLON
      COMMON/DIMEN/NRS,NCS,XROW,XCOL
      COMMON /GRD2MDSET/INC,LAS,LAN,LOE,LOW
      DATA MISG/Z'80808080'/
C
C  OPEN UP MD FILE TO BE WRITTEN TO
C
      IOK=MDOPEN(IMD,2)
      IF(IOK.LT.0) THEN
        PRINT *,'Trouble opening MD file; IOK= ',IOK
        GOTO 999
      ENDIF
C
C  SET UP LAT/LON AND INC VARIABLES
C
      NLA=(LAN-LAS)*10/INC + 1
      LLOW=LOW
C
C  ACCOUNT FOR GREENWICH
C
      IF (LLOW.LT.LOE) LLOW=LLOW+360
      NLO=(LLOW-LOE)*10/INC + 1
      JINC=INC*1000
C
C  CHANGE LONGITUDES TO MCIDAS CONVENTION (+W -E)
C
      IF (LOW.GT.180) LOW=LOW-360
      IF (LOE.GT.180) LOE=LOE-360
      IREC(1)=IDAY
      IREC(2)=ITIM
      call sdest('output day= ',IREC(1))
      call sdest('output time= ',IREC(2))
C
C  FIND LOCATION OF EACH KEY IN SCHEMA OF MD FILE
C
      ISTAT=MDKEYS(IMD,NKYS,CKYS,ISCALE,IUNIT,LOCS)
      IF (ISTAT .LT. 0) THEN
        PRINT *,'Unable to execute MDKEYS'
        GOTO 999
      ENDIF
C
C  PROCESS GRIDDED DATA
C
      DO 190 NG=IG1,IG2
        L=0
        CALL IGGET(IGRD,NG,70000,DUMMY,NROW,NCOL,HEADER)
        NRS=NROW
        NCS=NCOL
        NPT=NROW*NCOL
        PARM=CLIT(HEADER(7))
        PLEV=HEADER(10)
        IGVSCA=HEADER(8)
C
C  DETERMINE LOCATION OF GRID RELATIVE TO LIST OF KEYS
C
        I=0
        DO 150 IX=1,NPRM
        IF(CGKYS(IX).EQ.'T   ') NT=IGLVS(IX)
          DO 145 IY=1,IGLVS(IX)
C!            I=((IX-1)*IGLVS(IX))+IY
            I=I+1
            IF (PARM .EQ. CGKYS(IX) .AND. PLEV .EQ. NLVL(IX,IY)) THEN
              IF (IGOT(I) .NE. 1) THEN
                L=I
              ENDIF
            ENDIF
  145     CONTINUE
  150   CONTINUE
        IF (L .NE. 0) THEN
          SCL=10.**IGVSCA
          IGOT(L)=1
          CALL GRDDEF(HEADER)
C
C  STORE DATA IN MDFILE ARRAY FVAL
C
          NLAT=LAN*10000
          DO 180 N=1,NLA
            LON=LOW*10000
            FLAT=.0001*NLAT
            DO 170 M=1,NLO
              FLON=.0001*LON
              IF (FLON .GT. 180.) FLON=FLON-360.
              CALL TVALUEX(DUMMY,SCL,VAL)
              FVAL(L,M,N)=VAL
C
C  CONVERT RH TO DEWPOINT TEMPERATURE
C
              IF (PARM .EQ. 'RH  ')THEN
                IF (VAL .NE. 999999.) THEN
C
C  FOLLOWING CALCULATION OF DEWPOINT ASSUMES T GRIDS PRECEDE RH GRIDS
C   AND ARE IN SAME ORDER
C
                  TEM=FVAL(L-NT,M,N)
                  IF (TEM .EQ.999999.) THEN
                    FVAL(L,M,N)=999999.
                  ELSE 
                    P=PLEV
                    W=.01*WSAT(P,TEM)*VAL
                    FVAL(L,M,N)=DEWPT(P,TEM,W)
                  ENDIF
                ENDIF
              ENDIF
              LON=LON-JINC
  170       CONTINUE
            NLAT=NLAT-JINC
  180     CONTINUE
        ENDIF
  190 CONTINUE
C
C  MAKE CORRECTION TO WINDS ON POLAR STEREOGRAPHIC GRID
C
      CALL IGGET(IGRD,1,10000,DUMMY,NROW,NCOL,HEADER)
      IF (PARM.EQ.'U   '.AND.TYPE.EQ.2) THEN
        CALL GRDDEF(HEADER)
        DO 225 K=1,NWGRD/2
          INDEX1=NTGRD+K
          INDEX2=INDEX1+NWGRD/2
          NLAT=LAN*10000
          DO 220 N=1,NLA
            LON=LOW*10000
            FLAT=.0001*NLAT
            DO 210 M=1,NLO
              FLON=.0001*LON
C  NEED TO CALL VALUEX TO SET XROW AND XCOL
              CALL TVALUEX(DUMMY,SCL,VAL)
              VAL1=FVAL(INDEX1,M,N)
              VAL2=FVAL(INDEX2,M,N)
              CALL WINCOR(XCOL,XROW,HEADER,VAL1,VAL2,SPEED,DIRR,ULL,VLL)
              FVAL(INDEX1,M,N)=ULL
              FVAL(INDEX2,M,N)=VLL
              LON=LON-JINC
  210       CONTINUE
            NLAT=NLAT-JINC
  220     CONTINUE
  225   CONTINUE
      ENDIF
C
C  PREPARE DATA FOR OUTPUT
C
      NN=0
      NLAT=LAN*10000
      DO N=1,NLA
        FLAT=.0001*NLAT
        IREC(3)=NLAT
C
C  WRITE ROW HEADER
C
        NN=NN+1
        IOK=MDPUT(IMD,NN,0,IREC)
        IF (IOK.LT.0) THEN
          PRINT *,'CANNOT WRITE ROW HEADER'
          GOTO 999
        ENDIF
        LON=LOW*10000
        MM=0
        DO M=1,NLO
          MM=MM+1
          IF (NN .EQ. 1) THEN
C
C  WRITE COL HEADER
C
            IREC(4) = LON
            IF (IREC(4).LE.-1800000) IREC(4)=IREC(4)+3600000
            IF (IREC(4).GT.1800000) IREC(4)=IREC(4)-3600000
            IF (IREC(4).GT.1800000) IREC(4)=IREC(4)-3600000
            IOK=MDPUT(IMD,0,MM,IREC)
            IF (IOK.LT.0) THEN
              PRINT *,'CANNOT WRITE COL HEADER'
              GOTO 999
            ENDIF
          ENDIF
C
C  ZERO OUT DATA
C
          DO  K=5,400
            IREC(K)=MISG
          ENDDO
C
C  GO THROUGH ARRAY ONCE MORE TO ESTABLISH OUTPUT SCALING
C
          DO  L=1,NKYS
            IVAL=MISG
            KK=LOCS(L)
            IF (KK .NE. -1) THEN
              IF (IGOT(L) .NE. O) THEN
                VAL=FVAL(L,M,N)
                IF (VAL .NE. 999999.) THEN
                  SCL=10.**ISCALE(L)
                  IVAL=VAL*SCL
                 ENDIF
              ELSE
                PRINT *,'DO NOT HAVE ALL DATA FOR GUESS PROFILES'
                GOTO 999
              ENDIF
            ENDIF
            IREC(KK)=IVAL
          ENDDO
C
C  WRITE OUT DATA
C
          IOK=MDPUT(IMD,NN,MM,IREC)
          IF (IOK.LT.0) THEN
            PRINT *,'CANNOT WRITE DATA VECTOR'
            GOTO 999
          ENDIF
          LON=LON-JINC
        ENDDO
        NLAT=NLAT-JINC
      ENDDO
  300 CONTINUE
C
  999 RETURN
      END
